"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var http = require("http");
var fs = require("fs");
var path = require("path");
var handlers = require("./handlers");
var auth_1 = require("../../src/auth");
var args_1 = require("./args");
var config_1 = require("../../src/config");
var db_1 = require("../../src/db");
var configPath = '/usr/local/etc/cic-meta';
var argv = args_1.standardArgs.argv;
if (argv['config'] !== undefined) {
    configPath = argv['config'];
}
var config = new config_1.Config(configPath, argv['env-prefix']);
config.process();
console.debug(config.toString());
var fp = path.join(config.get('PGP_EXPORTS_DIR'), config.get('PGP_PUBLICKEY_ACTIVE_FILE'));
var pubksa = fs.readFileSync(fp, 'utf-8');
fp = path.join(config.get('PGP_EXPORTS_DIR'), config.get('PGP_PRIVATEKEY_FILE'));
var pksa = fs.readFileSync(fp, 'utf-8');
var dbConfig = {
    'name': config.get('DATABASE_NAME'),
    'user': config.get('DATABASE_USER'),
    'password': config.get('DATABASE_PASSWORD'),
    'host': config.get('DATABASE_HOST'),
    'port': config.get('DATABASE_PORT'),
    'engine': config.get('DATABASE_ENGINE')
};
var db = undefined;
if (config.get('DATABASE_ENGINE') == 'sqlite') {
    db = new db_1.SqliteAdapter(dbConfig);
}
else if (config.get('DATABASE_ENGINE') == 'postgres') {
    db = new db_1.PostgresAdapter(dbConfig);
}
else {
    throw 'database engine ' + config.get('DATABASE_ENGINE') + 'not implemented';
}
var signer = undefined;
var keystore = new auth_1.PGPKeyStore(config.get('PGP_PASSPHRASE'), pksa, pubksa, pubksa, pubksa, function () {
    keysLoaded();
});
function keysLoaded() {
    signer = new auth_1.PGPSigner(keystore);
    prepareServer();
}
function migrateDatabase(cb) {
    return __awaiter(this, void 0, void 0, function () {
        var sql;
        return __generator(this, function (_a) {
            try {
                sql = "SELECT 1 FROM store;";
                db.query(sql, function (e, rs) {
                    if (e === null || e === undefined) {
                        cb();
                        return;
                    }
                    console.warn('db check for table "store" fail', e);
                    console.debug('using schema path', config.get('DATABASE_SCHEMA_SQL_PATH'));
                    var sql = fs.readFileSync(config.get('DATABASE_SCHEMA_SQL_PATH'), 'utf-8');
                    db.query(sql, function (e, rs) {
                        if (e !== undefined && e !== null) {
                            console.error('db initialization fail', e);
                            return;
                        }
                        cb();
                    });
                });
            }
            catch (e) {
                console.warn('table store does not exist', e);
            }
            return [2 /*return*/];
        });
    });
}
function prepareServer() {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, migrateDatabase(startServer)];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    });
}
function startServer() {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            http.createServer(processRequest).listen(config.get('SERVER_PORT'));
            return [2 /*return*/];
        });
    });
}
var re_digest = /^\/([a-fA-F0-9]{64})\/?$/;
function parseDigest(url) {
    var digest_test = url.match(re_digest);
    if (digest_test === null) {
        throw 'invalid digest';
    }
    return digest_test[1].toLowerCase();
}
function processRequest(req, res) {
    return __awaiter(this, void 0, void 0, function () {
        var digest, headers, mergeHeader, mod, data;
        var _this = this;
        return __generator(this, function (_a) {
            digest = undefined;
            headers = {
                "Access-Control-Allow-Origin": "*",
                "Access-Control-Allow-Methods": "OPTIONS, POST, GET, PUT",
                "Access-Control-Max-Age": 2592000,
                "Access-Control-Allow-Headers": 'Access-Control-Allow-Origin, Content-Type, x-cic-automerge'
            };
            if (req.method === "OPTIONS") {
                res.writeHead(200, headers);
                res.end();
                return [2 /*return*/];
            }
            if (!['PUT', 'GET', 'POST'].includes(req.method)) {
                res.writeHead(405, { "Content-Type": "text/plain" });
                res.end();
                return [2 /*return*/];
            }
            try {
                digest = parseDigest(req.url);
            }
            catch (e) {
                console.error('digest error: ' + e);
                res.writeHead(400, { "Content-Type": "text/plain" });
                res.end();
                return [2 /*return*/];
            }
            mergeHeader = req.headers['x-cic-automerge'];
            mod = req.method.toLowerCase() + ":automerge:";
            switch (mergeHeader) {
                case "client":
                    mod += "client"; // client handles merges
                    break;
                case "server":
                    mod += "server"; // server handles merges
                    break;
                default:
                    mod += "none"; // merged object only (get only)
            }
            data = '';
            req.on('data', function (d) {
                data += d;
            });
            req.on('end', function () { return __awaiter(_this, void 0, void 0, function () {
                var content, contentType, r, _a, e_1, responseContentLength;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            console.debug('mode', mod);
                            content = '';
                            contentType = 'application/json';
                            console.debug('handling data', data);
                            r = undefined;
                            _b.label = 1;
                        case 1:
                            _b.trys.push([1, 14, , 15]);
                            _a = mod;
                            switch (_a) {
                                case 'put:automerge:client': return [3 /*break*/, 2];
                                case 'get:automerge:client': return [3 /*break*/, 4];
                                case 'post:automerge:server': return [3 /*break*/, 6];
                                case 'put:automerge:server': return [3 /*break*/, 8];
                                case 'get:automerge:none': return [3 /*break*/, 10];
                            }
                            return [3 /*break*/, 12];
                        case 2: return [4 /*yield*/, handlers.handleClientMergePut(data, db, digest, keystore, signer)];
                        case 3:
                            r = _b.sent();
                            if (r == false) {
                                res.writeHead(403, { "Content-Type": "text/plain" });
                                res.end();
                                return [2 /*return*/];
                            }
                            return [3 /*break*/, 13];
                        case 4: return [4 /*yield*/, handlers.handleClientMergeGet(db, digest, keystore)];
                        case 5:
                            content = _b.sent();
                            return [3 /*break*/, 13];
                        case 6: return [4 /*yield*/, handlers.handleServerMergePost(data, db, digest, keystore, signer)];
                        case 7:
                            content = _b.sent();
                            return [3 /*break*/, 13];
                        case 8: return [4 /*yield*/, handlers.handleServerMergePut(data, db, digest, keystore, signer)];
                        case 9:
                            r = _b.sent();
                            if (r == false) {
                                res.writeHead(403, { "Content-Type": "text/plain" });
                                res.end();
                                return [2 /*return*/];
                            }
                            return [3 /*break*/, 13];
                        case 10: return [4 /*yield*/, handlers.handleNoMergeGet(db, digest, keystore)];
                        case 11:
                            r = _b.sent();
                            if (r == false) {
                                res.writeHead(404, { "Content-Type": "text/plain" });
                                res.end();
                                return [2 /*return*/];
                            }
                            content = r;
                            return [3 /*break*/, 13];
                        case 12:
                            res.writeHead(400, { "Content-Type": "text/plain" });
                            res.end();
                            return [2 /*return*/];
                        case 13: return [3 /*break*/, 15];
                        case 14:
                            e_1 = _b.sent();
                            console.error('fail', mod, digest, e_1);
                            res.writeHead(500, { "Content-Type": "text/plain" });
                            res.end();
                            return [2 /*return*/];
                        case 15:
                            if (content === undefined) {
                                console.error('empty onctent', data);
                                res.writeHead(400, { "Content-Type": "text/plain" });
                                res.end();
                                return [2 /*return*/];
                            }
                            responseContentLength = (new TextEncoder().encode(content)).length;
                            res.writeHead(200, {
                                "Access-Control-Allow-Origin": "*",
                                "Content-Type": contentType,
                                "Content-Length": responseContentLength
                            });
                            res.write(content);
                            res.end();
                            return [2 /*return*/];
                    }
                });
            }); });
            return [2 /*return*/];
        });
    });
}
