"use strict";
exports.__esModule = true;
exports.PGPKeyStore = exports.PGPSigner = void 0;
var pgp = require("openpgp");
var PGPKeyStore = /** @class */ (function () {
    function PGPKeyStore(passphrase, pkArmor, pubkActiveArmor, pubkTrustedArmor, pubkEncryptArmor, onload) {
        if (onload === void 0) { onload = function (ks) { }; }
        this.pubk = {
            active: [],
            trusted: [],
            encrypt: []
        };
        this.loads = 0x00;
        this.loadsTarget = 0x0f;
        this._readKey(pkArmor, undefined, 1, passphrase);
        this._readKey(pubkActiveArmor, 'active', 2);
        this._readKey(pubkTrustedArmor, 'trusted', 4);
        this._readKey(pubkEncryptArmor, 'encrypt', 8);
        this.onload = onload;
    }
    PGPKeyStore.prototype._readKey = function (a, x, n, pass) {
        var _this = this;
        pgp.key.readArmored(a).then(function (k) {
            if (pass !== undefined) {
                _this.pk = k.keys[0];
                _this.pk.decrypt(pass).then(function () {
                    _this.fingerprint = _this.pk.getFingerprint();
                    console.log('private key (sign)', _this.fingerprint);
                    _this._registerLoad(n);
                });
            }
            else {
                _this.pubk[x] = k.keys;
                k.keys.forEach(function (pubk) {
                    console.log('public key (' + x + ')', pubk.getFingerprint());
                });
                _this._registerLoad(n);
            }
        });
    };
    PGPKeyStore.prototype._registerLoad = function (b) {
        this.loads |= b;
        if (this.loads == this.loadsTarget) {
            this.onload(this);
        }
    };
    PGPKeyStore.prototype.getTrustedKeys = function () {
        return this.pubk['trusted'];
    };
    PGPKeyStore.prototype.getTrustedActiveKeys = function () {
        return this.pubk['active'];
    };
    PGPKeyStore.prototype.getEncryptKeys = function () {
        return this.pubk['encrypt'];
    };
    PGPKeyStore.prototype.getPrivateKey = function () {
        return this.pk;
    };
    PGPKeyStore.prototype.getFingerprint = function () {
        return this.fingerprint;
    };
    return PGPKeyStore;
}());
exports.PGPKeyStore = PGPKeyStore;
var PGPSigner = /** @class */ (function () {
    function PGPSigner(keyStore) {
        this.engine = 'pgp';
        this.algo = 'sha256';
        this.keyStore = keyStore;
        this.onsign = function (string) { };
        this.onverify = function (boolean) { };
    }
    PGPSigner.prototype.fingerprint = function () {
        return this.keyStore.getFingerprint();
    };
    PGPSigner.prototype.prepare = function (material) {
        this.dgst = material.digest();
        return true;
    };
    PGPSigner.prototype.verify = function (digest, signature) {
        var _this = this;
        pgp.signature.readArmored(signature.data).then(function (s) {
            var opts = {
                message: pgp.cleartext.fromText(digest),
                publicKeys: _this.keyStore.getTrustedKeys(),
                signature: s
            };
            pgp.verify(opts).then(function (v) {
                var i = 0;
                for (i = 0; i < v.signatures.length; i++) {
                    var s_1 = v.signatures[i];
                    if (s_1.valid) {
                        _this.onverify(s_1);
                        return;
                    }
                }
                console.error('checked ' + i + ' signature(s) but none valid');
                _this.onverify(false);
            });
        })["catch"](function (e) {
            console.error(e);
            _this.onverify(false);
        });
    };
    PGPSigner.prototype.sign = function (digest) {
        var _this = this;
        var m = pgp.cleartext.fromText(digest);
        var pk = this.keyStore.getPrivateKey();
        var opts = {
            message: m,
            privateKeys: [pk],
            detached: true
        };
        pgp.sign(opts).then(function (s) {
            _this.signature = {
                engine: _this.engine,
                algo: _this.algo,
                data: s.signature,
                // TODO: fix for browser later
                digest: digest
            };
            _this.onsign(_this.signature);
        })["catch"](function (e) {
            console.error(e);
            _this.onsign(undefined);
        });
    };
    return PGPSigner;
}());
exports.PGPSigner = PGPSigner;
