"use strict";
exports.__esModule = true;
exports.PostgresAdapter = exports.SqliteAdapter = void 0;
var pg = require("pg");
var sqlite = require("sqlite3");
var re_creatematch = /^(CREATE)/i;
var re_getmatch = /^(SELECT)/i;
var re_setmatch = /^(INSERT|UPDATE)/i;
var SqliteAdapter = /** @class */ (function () {
    function SqliteAdapter(dbConfig, callback) {
        this.db = new sqlite.Database(dbConfig.name); //, callback);
    }
    SqliteAdapter.prototype.query = function (s, callback) {
        var local_callback = function (e, rs) {
            var r = undefined;
            if (rs !== undefined) {
                r = {
                    rowCount: rs.length,
                    rows: rs
                };
            }
            callback(e, r);
        };
        if (s.match(re_getmatch)) {
            this.db.all(s, local_callback);
        }
        else if (s.match(re_setmatch)) {
            this.db.run(s, local_callback);
        }
        else if (s.match(re_creatematch)) {
            this.db.run(s, callback);
        }
        else {
            throw 'unhandled query';
        }
    };
    SqliteAdapter.prototype.close = function () {
        this.db.close();
    };
    return SqliteAdapter;
}());
exports.SqliteAdapter = SqliteAdapter;
var PostgresAdapter = /** @class */ (function () {
    function PostgresAdapter(dbConfig) {
        var o = dbConfig;
        o['database'] = o.name;
        this.db = new pg.Pool(o);
        return this.db;
    }
    PostgresAdapter.prototype.query = function (s, callback) {
        this.db.query(s, function (e, rs) {
            var r = {
                length: rs.rowCount
            };
            rs.length = rs.rowCount;
            if (e === undefined) {
                e = null;
            }
            console.debug(e, rs);
            callback(e, rs);
        });
    };
    PostgresAdapter.prototype.close = function () {
        this.db.end();
    };
    return PostgresAdapter;
}());
exports.PostgresAdapter = PostgresAdapter;
