"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fromIndexKey = exports.toIndexKey = exports.Dispatcher = void 0;
var uuid_1 = require("uuid");
function toIndexKey(id) {
    var d = Date.now();
    return d + '_' + id + '_' + uuid_1.v4();
}
exports.toIndexKey = toIndexKey;
var _re_indexKey = /^\d+_(.+)_[-\d\w]+$/;
function fromIndexKey(s) {
    var m = s.match(_re_indexKey);
    if (m === null) {
        throw 'Invalid index key';
    }
    return m[1];
}
exports.fromIndexKey = fromIndexKey;
var Dispatcher = /** @class */ (function () {
    function Dispatcher(store, syncer) {
        this.idx = new Array();
        this.syncer = syncer;
        this.store = store;
    }
    Dispatcher.prototype.isDirty = function () {
        return this.idx.length > 0;
    };
    Dispatcher.prototype.add = function (id, item) {
        var v = item.toJSON();
        var k = toIndexKey(id);
        this.store.put(k, v, true);
        localStorage.setItem(k, v);
        this.idx.push(k);
        return k;
    };
    Dispatcher.prototype.sync = function (offset) {
        var _this = this;
        var i = 0;
        this.idx.forEach(function (k) {
            var v = localStorage.getItem(k);
            var k_id = fromIndexKey(k);
            _this.syncer.pub(v); // this must block until guaranteed delivery
            localStorage.removeItem(k);
            i++;
        });
        return i;
    };
    return Dispatcher;
}());
exports.Dispatcher = Dispatcher;
